@@ -7,7 +7,7 @@
 
 /* playlist data */
 enum {
-    PLAYLISTS_COL_NUM = 0,
+    PLAYLISTS_COL_ID = 0,
     PLAYLISTS_COL_NAME,
     PLAYLISTS_COL_AUTHOR,
     PLAYLISTS_NUM_COLS
