@@ -1,11 +1,12 @@
 #include <glib.h>
 /* Include the prototypes for GConf client functions. */
 #include <gconf/gconf-client.h>
+#include <config.h>
 
+#include "userdata.h"
 #include "settings.h"
 
-#define MSPOTIFY_APP_NAME "yaspot"
-#define MSPOTIFY_CONF_ROOT "/apps/Maemo/" MSPOTIFY_APP_NAME "/"
+#define YASPOT_CONF_ROOT "/apps/Maemo/" PACKAGE "/"
 
 static gint store_string(const gchar *path, const gchar *str) {
     GConfClient *gcClient = NULL;
@@ -104,39 +105,56 @@ end:
 }
 
 gchar* yaspot_settings_username() {
-    return fetch_string(MSPOTIFY_CONF_ROOT "username");
+    return fetch_string(YASPOT_CONF_ROOT "username");
 }
 
 gchar* yaspot_settings_password() {
-    return fetch_string(MSPOTIFY_CONF_ROOT "password");
+    return fetch_string(YASPOT_CONF_ROOT "password");
 }
 
 int yaspot_settings_set_username(const gchar *username) {
     if (!username)
         return -1;
-    return store_string(MSPOTIFY_CONF_ROOT "username", username);
+    return store_string(YASPOT_CONF_ROOT "username", username);
 }
 
 int yaspot_settings_set_password(const gchar *password) {
     if (!password)
         return -1;
-    return store_string(MSPOTIFY_CONF_ROOT "password", password);
+    return store_string(YASPOT_CONF_ROOT "password", password);
 }
 
 int yaspot_settings_save_credentials(int *num) {
-    return fetch_int(MSPOTIFY_CONF_ROOT "save_credentials", num);
+    return fetch_int(YASPOT_CONF_ROOT "save_credentials", num);
 }
 
 int yaspot_settings_set_save_credentials(int save_credentials) {
-    return store_int(MSPOTIFY_CONF_ROOT "save_credentials", save_credentials);
+    return store_int(YASPOT_CONF_ROOT "save_credentials", save_credentials);
 }
 
 int yaspot_settings_high_bitrate(int *num) {
-    return fetch_int(MSPOTIFY_CONF_ROOT "high_bitrate", num);
+    return fetch_int(YASPOT_CONF_ROOT "high_bitrate", num);
 }
 
 int yaspot_settings_set_high_bitrate(int high_bitrate) {
-    return store_int(MSPOTIFY_CONF_ROOT "high_bitrate", high_bitrate);
+    return store_int(YASPOT_CONF_ROOT "high_bitrate", high_bitrate);
+}
+
+/* RESTORED SETTINGS with yaspot_settings_restore */
+
+int yaspot_settings_autoshow_queue(gint autoshow_queue) {
+    return store_int(YASPOT_CONF_ROOT "autoshow_queue", autoshow_queue);
+}
+
+int yaspot_settings_get_autoshow_queue(gint *autoshow_queue) {
+    return fetch_int(YASPOT_CONF_ROOT "autoshow_queue", autoshow_queue);
+}
+
+void yaspot_settings_restore(userdata_t *u) {
+    gint var;
+
+    if (!yaspot_settings_get_autoshow_queue(&var))
+        u->settings.autoshow_queue = var;
 }
 
 // vim: expandtab shiftwidth=4
