@@ -5,6 +5,8 @@
 
 GtkWidget* yaspot_view_main_new();
 void yaspot_show_main(userdata_t *u);
+void yaspot_display_login(userdata_t *u, gint *quit);
+void yaspot_main_set_sensitive(userdata_t *u, gboolean sensitive);
 
 #endif
 
