@@ -10,6 +10,8 @@
 #include "glue.h"
 #include "app-menu.h"
 #include "views.h"
+#include "view-main.h"
+#include "history.h"
 
 typedef struct progress_t {
     //GtkWidget *window;
@@ -30,6 +32,7 @@ typedef struct yaspot_album_t {
     GtkWidget *bin;
     GtkTreeView *tree;
     GtkWidget *image;
+    userdata_t *u;
 } yaspot_album_t;
 
 typedef struct yaspot_view_t {
@@ -54,6 +57,11 @@ typedef struct yaspot_view_t {
     yaspot_glue_response_t *r;
 } yaspot_view_t;
 
+typedef struct get_async_t {
+    userdata_t *u;
+    GtkWidget *window;
+} get_async_t;
+
 static gint artist_view_block = FALSE;
 
 /* *** common *** */
@@ -62,7 +70,7 @@ static const gint32 STOP_TIMER_LOOP = -1;
 static const gint32 UPDATE_SELECTION = -2;
 static const gint32 UPDATE_PLAYPAUSE = -3;
 
-static void state_changed_cb(gboolean playing, gboolean paused, gpointer data);
+static void state_changed_cb(yaspot_state_t state, yaspot_playback_state_t pb_state, gpointer data);
 
 static yaspot_view_t* view_new(userdata_t *u) {
     yaspot_view_t *v;
@@ -79,11 +87,12 @@ static void view_free(yaspot_view_t *v) {
     g_free(v);
 }
 
-static void destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
+static gboolean destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
     yaspot_view_t *v = (yaspot_view_t*)data;
     g_print("destroy view %p\n", v);
-    gtk_widget_destroy(widget);
     view_free(v);
+
+    return FALSE;
 }
 
 static void free_artist(gpointer data, gpointer userdata) {
@@ -92,17 +101,26 @@ static void free_artist(gpointer data, gpointer userdata) {
     g_free(a);
 }
 
-static void artist_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
+static gboolean artist_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
     yaspot_view_t *v = (yaspot_view_t*)data;
     g_print("destroy artist view %p\n", v);
     g_list_foreach(v->artist_view, free_artist, NULL);
     g_list_free(v->artist_view);
     g_print("and now foo\n");
     destroy(widget, event, data);
+
+    return FALSE;
 }
 
-static void queue_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
+static gboolean queue_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
     yaspot_view_t *v = (yaspot_view_t*)data;
+    userdata_t *u = v->u;
+
+    g_assert(widget);
+    g_assert(event);
+    g_assert(data);
+
+    yaspot_glue_del_state_cb(v->u->session, state_changed_cb, v);
 
     if (v->timer) {
         yaspot_glue_set_timer_cb(v->u->session, NULL, NULL);
@@ -113,9 +131,11 @@ static void queue_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
         g_async_queue_unref(v->timer_queue);
     }
 
-    yaspot_glue_del_state_cb(v->u->session, state_changed_cb, v);
-
     destroy(widget, event, data);
+
+    u->queue_visible = FALSE;
+
+    return FALSE;
 }
 
 static yaspot_view_t* new_basic_view(yaspot_view_t *v) {
@@ -154,10 +174,10 @@ static void row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewC
     if (value > -1) {
         g_print("playlist: row_activated, row %d column %s\n", value, gtk_tree_view_column_get_title(column));
 
-        if (column == gtk_tree_view_get_column(treeview, PLAYLIST_COL_TITLE))
+        if (column == gtk_tree_view_get_column(treeview, PLAYLIST_COL_TITLE)) {
             yaspot_glue_play(v->r, value);
-        else if (column == gtk_tree_view_get_column(treeview, PLAYLIST_COL_ALBUM))
-            yaspot_show_album(v->u, v->r, value);
+        } else if (column == gtk_tree_view_get_column(treeview, PLAYLIST_COL_ALBUM))
+            yaspot_show_album(v->u, v->r, value, v->window);
         else if (column == gtk_tree_view_get_column(treeview, PLAYLIST_COL_ARTIST))
             yaspot_show_artist(v->u, v->r, value, v->window);
     }
@@ -204,7 +224,7 @@ static void lists_row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTre
     yaspot_view_t *v = (yaspot_view_t*)data;
     GtkTreeIter iter;
     GtkTreeModel *model;
-    gint value;
+    gchar *value;
 
     model = gtk_tree_view_get_model(treeview);
     if (!gtk_tree_model_get_iter(model, &iter, path)) {
@@ -212,27 +232,20 @@ static void lists_row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTre
         return;
     }
 
-    gtk_tree_model_get(model, &iter, PLAYLIST_COL_TRACK, &value, -1);
+    gtk_tree_model_get(model, &iter, PLAYLISTS_COL_ID, &value, -1);
 
-    if (value > -1) {
-        g_print("load playlist %d\n", value);
+    if (value) {
+        g_print("load playlist %s\n", value);
 
-        yaspot_show_stored_list(v->u, v->r, value);
+        yaspot_show_stored_list_by_id(v->u, value, v->window);
     }
 }
 
-void yaspot_show_stored_list(userdata_t *u, yaspot_glue_response_t *list, gint num) {
-
+static void setup_stored_list(userdata_t *u, yaspot_glue_response_t *r) {
     gchar *playlist_name;
     GtkListStore *model;
-    yaspot_glue_response_t *r;
     yaspot_view_t *v;
 
-    if (!(r = yaspot_glue_stored_playlist(list, num))) {
-        g_print("couldn't get stored playlist %d\n", num);
-        return;
-    }
-
     v = new_basic_view(view_new(u));
     v->r = r;
 
@@ -243,6 +256,7 @@ void yaspot_show_stored_list(userdata_t *u, yaspot_glue_response_t *list, gint n
 
     if (playlist_name) {
         gtk_window_set_title(GTK_WINDOW(v->window), playlist_name);
+        yaspot_history_add(u->history, HISTORY_PLAYLIST, playlist_name, yaspot_glue_response_id(r));
         g_free(playlist_name);
     } else {
         gtk_window_set_title(GTK_WINDOW(v->window), _("Playlist"));
@@ -256,23 +270,63 @@ void yaspot_show_stored_list(userdata_t *u, yaspot_glue_response_t *list, gint n
     gtk_widget_show_all(v->window);
 }
 
-void yaspot_show_stored_playlists(userdata_t *u) {
-    GtkListStore *model;
-    yaspot_view_t *v;
-    yaspot_glue_response_t *r;
+static void show_stored_list_reply(yaspot_glue_t *t, gpointer response, gpointer userdata) {
+    yaspot_glue_response_t *r = (yaspot_glue_response_t*)response;
+    get_async_t *g = (get_async_t*)userdata;
 
-    g_print("get playlists\n");
-    if (!(r = yaspot_glue_stored_lists(u->session))) {
-        g_print("couldn't get stored playlists\n");
+    if (!r) {
+        g_print("couldn't get stored playlist\n");
+        g_free(g);
         return;
     }
-    g_print("get playlists done\n");
+
+    gdk_threads_enter();
+    setup_stored_list(g->u, r);
+    gdk_threads_leave();
+
+    if (g->window) {
+        gdk_threads_enter();
+        hildon_gtk_window_set_progress_indicator(GTK_WINDOW(g->window), FALSE);
+        gdk_threads_leave();
+    }
+
+    g_free(g);
+}
+
+void yaspot_show_stored_list_by_id(userdata_t *u, const gchar *id, GtkWidget *window) {
+    g_assert(u);
+    g_assert(id);
+
+    get_async_t *g = g_new0(get_async_t, 1);
+    g->u = u;
+    g->window = window;
+
+    if (g->window)
+        hildon_gtk_window_set_progress_indicator(GTK_WINDOW(g->window), TRUE);
+
+    yaspot_glue_stored_playlist_by_id(u->session, show_stored_list_reply, id, g);
+}
+
+static void show_stored_playlists(userdata_t *u) {
+    GtkListStore *model;
+    const yaspot_playlist_entry_t *e;
+    void *state = NULL;
+    int i = 0;
+    yaspot_view_t *v;
 
     v = new_playlist_view(view_new(u));
-    v->r = r;
 
     model = GTK_LIST_STORE(gtk_tree_view_get_model(v->treeview));
-    yaspot_glue_response_to_model(r, model);
+
+    gtk_list_store_clear(model);
+
+    while ((e = yaspot_playlist_iterate(v->u->session, &state))) {
+        gtk_list_store_insert_with_values(model, NULL, i++,
+                PLAYLISTS_COL_ID, e->id,
+                PLAYLISTS_COL_NAME, e->name,
+                PLAYLISTS_COL_AUTHOR, e->author,
+                -1);
+    }
 
     g_signal_connect(G_OBJECT(v->treeview), "row-activated", G_CALLBACK(lists_row_activated), v);
 
@@ -282,6 +336,44 @@ void yaspot_show_stored_playlists(userdata_t *u) {
                      G_CALLBACK(destroy), v);
 
     gtk_widget_show_all(v->window);
+
+    yaspot_main_set_sensitive(u, TRUE);
+}
+
+static void show_stored_playlists_cb(yaspot_glue_t *t, gpointer reply, gpointer userdata) {
+    userdata_t *u = (userdata_t*)userdata;
+    gint success;
+
+    g_assert(t);
+    g_assert(u);
+
+    success = GPOINTER_TO_INT(reply);
+
+    if (!success) {
+        g_print("couldn't get stored playlists\n");
+        gdk_threads_enter();
+        yaspot_main_set_sensitive(u, TRUE);
+        gdk_threads_leave();
+
+        return;
+    }
+
+    g_print("get playlists done\n");
+
+    gdk_threads_enter();
+    show_stored_playlists(u);
+    gdk_threads_leave();
+}
+
+void yaspot_show_stored_playlists(userdata_t *u) {
+    g_print("get playlists\n");
+
+    if (yaspot_playlist_length(u->session) == 0) {
+        yaspot_main_set_sensitive(u, FALSE);
+        yaspot_playlist_reload(u->session, show_stored_playlists_cb, u);
+    } else {
+        show_stored_playlists(u);
+    }
 }
 
 void yaspot_show_search(userdata_t *u, const gchar *search) {
@@ -296,7 +388,7 @@ void yaspot_show_search(userdata_t *u, const gchar *search) {
         return;
     }
 
-
+    yaspot_history_add(u->history, HISTORY_SEARCH, search, search);
     v = new_basic_view(view_new(u));
     v->r = r;
 
@@ -535,6 +627,7 @@ gpointer artist_thread(gpointer data) {
         yaspot_album_t *a = g_new0(yaspot_album_t, 1);
         v->artist_view = g_list_append(v->artist_view, a);
 
+        a->u = v->u;
         a->r = new_r;
         a->vbox_all = gtk_vbox_new(FALSE, 10);
         a->hbox = gtk_hbox_new(FALSE, 10);
@@ -663,14 +756,8 @@ static yaspot_view_t* new_album_view(yaspot_view_t *v) {
     return v;
 }
 
-void yaspot_show_album(userdata_t *u, yaspot_glue_response_t *list, gint num) {
+static void setup_album(userdata_t *u, yaspot_glue_response_t *r) {
     yaspot_view_t *v;
-    yaspot_glue_response_t *r;
-
-    if (!(r = yaspot_glue_album(list, num))) {
-        g_warning("show album failed\n");
-        return;
-    }
 
     v = new_album_view(view_new(u));
     v->r = r;
@@ -678,6 +765,7 @@ void yaspot_show_album(userdata_t *u, yaspot_glue_response_t *list, gint num) {
     gchar *label = yaspot_glue_response_get_name(r);
     if (label) {
         gtk_window_set_title(GTK_WINDOW(v->window), label);
+        yaspot_history_add(u->history, HISTORY_ALBUM, label, yaspot_glue_response_id(r));
         g_free(label);
     }
 
@@ -693,6 +781,57 @@ void yaspot_show_album(userdata_t *u, yaspot_glue_response_t *list, gint num) {
     gtk_widget_show_all(v->window);
 }
 
+static void show_album_reply(yaspot_glue_t *t, gpointer response, gpointer userdata) {
+    yaspot_glue_response_t *r = (yaspot_glue_response_t*)response;
+    get_async_t *g = (get_async_t*)userdata;
+
+    if (!r) {
+        g_print("couldn't get album\n");
+        g_free(g);
+        return;
+    }
+
+    gdk_threads_enter();
+    setup_album(g->u, r);
+    gdk_threads_leave();
+
+    if (g->window) {
+        gdk_threads_enter();
+        hildon_gtk_window_set_progress_indicator(GTK_WINDOW(g->window), FALSE);
+        gdk_threads_leave();
+    }
+
+    g_free(g);
+}
+
+void yaspot_show_album(userdata_t *u, yaspot_glue_response_t *list, gint num, GtkWidget *window) {
+    g_assert(u);
+    g_assert(list);
+
+    get_async_t *g = g_new0(get_async_t, 1);
+    g->u = u;
+    g->window = window;
+
+    if (g->window)
+        hildon_gtk_window_set_progress_indicator(GTK_WINDOW(g->window), TRUE);
+
+    yaspot_glue_album(list, show_album_reply, num, g);
+}
+
+void yaspot_show_album_by_id(userdata_t *u, const gchar *id, GtkWidget *window) {
+    g_assert(u);
+    g_assert(id);
+
+    get_async_t *g = g_new0(get_async_t, 1);
+    g->u = u;
+    g->window = window;
+
+    if (g->window)
+        hildon_gtk_window_set_progress_indicator(GTK_WINDOW(g->window), TRUE);
+
+    yaspot_glue_album_by_id(u->session, show_album_reply, id, g);
+}
+
 static void update_timer(yaspot_glue_t *t, gint now, gint length, gpointer userdata) {
     yaspot_view_t *v = (yaspot_view_t*)userdata;
     guint32 msg;
@@ -829,19 +968,19 @@ static void queue_selection_cb(GtkTreeSelection *selection, gpointer data) {
     }
 }
 
-static void state_changed_cb(gboolean playing, gboolean paused, gpointer data) {
+static void state_changed_cb(yaspot_state_t state, yaspot_playback_state_t pb_state, gpointer data) {
     yaspot_view_t *v = (yaspot_view_t*)data;
 
     g_assert(v);
 
     g_print("queue: state changed\n");
 
-    if (!playing)
+    if (!(pb_state & YASPOT_PLAYING))
         return;
 
     g_async_queue_push(v->timer_queue, GINT_TO_POINTER(UPDATE_PLAYPAUSE));
 
-    if (!paused)
+    if (!(pb_state & YASPOT_PAUSED))
         g_async_queue_push(v->timer_queue, GINT_TO_POINTER(UPDATE_SELECTION));
 }
 
@@ -857,6 +996,8 @@ void yaspot_show_queue(userdata_t *u) {
     GtkWidget *image_next;
     GtkWidget *image_pp;
 
+    u->queue_visible = TRUE;
+
     v = view_new(u);
     v->window = hildon_stackable_window_new();
     v->area = HILDON_PANNABLE_AREA( hildon_pannable_area_new() );
